class CurrentPos(object):

    # Class constructor
    def __init__(self, position_data):
        self.position = position_data.split(',')
        self.x = float(self.position[0])
        self.y = float(self.position[1])
        self.z = float(self.position[2])
        self.tx = float(self.position[3])
        self.ty = float(self.position[4])
        self.tz = float(self.position[5])

    def get_x(self):
        return self.x

    def get_y(self):
        return self.y

    def get_z(self):
        return self.z

    def get_tx(self):
        return self.tx

    def get_ty(self):
        return self.ty

    def get_tz(self):
        return self.tz



class Gripper(object):

    # Class constructor
    def __init__(self):
        self.is_closed = False
        self.is_acknowledge = False
        self.is_hit = False

    def set_closed_status(self, register_value):
        self.is_closed = True if int(register_value) == 32 else False

    def is_gripper_closed(self):
        return self.is_closed

    def set_acknowledge(self, register_value):
        self.is_acknowledge = True if register_value == '4' else False

    def is_acknowledge(self):
        return self.is_acknowledge

    def set_hit(self, register_value):
        self.is_hit = True if register_value == '4' else False

    def is_hit(self):
        return self.is_hit



"""
Commander class for linear moves
"""

from nx100_remote_control.module import Commands, Utils
import time


class LinearMove(object):

    # Class constructor
    def __init__(self):
        self.stopped = False

    def go(self, move_l, wait=True, poll_limit_seconds=30):
        """
        commands robot to move into position linear way

        :param move_l: MoveL object describing target position and move settings
        :param wait: wait for move to complete or not
        :param poll_limit_seconds: functions as a timeout of wait is enabled
        :return: boolean
        """
        self.stopped = False
        Commands.write_hold('0')  # disable hold if currently enabled
        Commands.write_linear_move(move_l=move_l)  # execute wanted move command
        current = 0
        if not wait:
            return True
        for x in range(poll_limit_seconds):
            if self.stopped:
                return False
            time.sleep(1)
            cp = Commands.read_current_specified_coordinate_system_position(  # returns CurrentPos object
                str(move_l.get_coordinate_specification()), '0'
            )
            if Utils.is_in_position(move_l, cp):
                return True
            else:
                current = current + 1
                if current == poll_limit_seconds:
                    return False

    def stop(self):
        """
        stop upper go function
        """
        self.stopped = True
        Commands.write_hold('1')  # only way to stop robot from executing move



import logging

CR = "\r"


# return byte length of command
def utf8len(s):
    return len(s.encode('utf-8'))


# get byte length for command data
def command_data_length(command):
    if len(command.data) == 0:
        return 0
    else:
        return utf8len(command.data + CR)


# just print some response details
def print_response_details(command_response):
    command_response_len = len(command_response)
    logging.info("Command response length: %d" % command_response_len)
    logging.info("Command response: " + command_response)


# decimal to binary
def decimal_to_binary(num):
    return bin(num)


# binary to decimal
def binary_to_decimal(binary):
    decimal, i, n = 0, 0, 0
    while binary != 0:
        dec = binary % 10
        decimal = decimal + dec * pow(2, i)
        binary = binary // 10
        i += 1
    return decimal


# clean response
def clean_response(response):
    return response \
        .replace('b\'', '') \
        .replace('\\r\'', '') \
        .replace('\\r\\n\'', '')


def equivalent(a, b, tolerance):
    return abs(a - b) < tolerance


# noinspection DuplicatedCode
def is_in_position(move_l, cp):
    t = 0.01  # tolerance x,y,z
    t2 = 0.1  # tolerance tx, ty, tz
    return equivalent(cp.get_x(), move_l.get_x(), t) and equivalent(cp.get_y(), move_l.get_y(), t) and equivalent(
        cp.get_z(), move_l.get_z(), t) and equivalent(cp.get_tx(), move_l.get_tx(), t2) \
           and equivalent(cp.get_ty(), move_l.get_ty(), t2) and equivalent(cp.get_tz(), move_l.get_tz(), t2)



class Status(object):

    # Class constructor
    def __init__(self, data_1, data_2):
        self.data_1 = list(data_1.replace('0b', ''))
        self.data_2 = list(data_2.replace('0b', ''))

    def is_command_remote(self):
        p = 0
        return (self.data_1[p] == '1') if len(self.data_1) > p else False

    def is_play(self):
        p = 1
        return (self.data_1[p] == '1') if len(self.data_1) > p else False

    def is_teach(self):
        p = 2
        return (self.data_1[p] == '1') if len(self.data_1) > p else False

    def is_safety_speed_operation(self):
        p = 3
        return (self.data_1[p] == '1') if len(self.data_1) > p else False

    def is_running(self):
        p = 4
        return (self.data_1[p] == '1') if len(self.data_1) > p else False

    def is_auto(self):
        p = 5
        return (self.data_1[p] == '1') if len(self.data_1) > p else False

    def is_one_cycle(self):
        p = 6
        return (self.data_1[p] == '1') if len(self.data_1) > p else False

    def is_step(self):
        p = 7
        return (self.data_1[p] == '1') if len(self.data_1) > p else False

    def is_servo_on(self):
        p = 1
        return (self.data_2[p] == '1') if len(self.data_2) > p else False

    def is_error_occurring(self):
        p = 2
        return (self.data_2[p] == '1') if len(self.data_2) > p else False

    def is_alarm_occurring(self):
        p = 3
        return (self.data_2[p] == '1') if len(self.data_2) > p else False

    def is_command_hold(self):
        p = 4
        return (self.data_2[p] == '1') if len(self.data_2) > p else False

    def is_external_hold(self):
        p = 5
        return (self.data_2[p] == '1') if len(self.data_2) > p else False

    def is_programming_pendant_hold(self):
        p = 6
        return (self.data_2[p] == '1') if len(self.data_2) > p else False



import time


class Time(object):

    # Class constructor
    def __init__(self, millis):
        self.millis = millis

    def set_time_now(self, millis):
        self.millis = millis

    def has_seconds_passed(self, seconds):
        return (self.millis + (seconds * 1000)) < self.get_current_millis()

    @staticmethod
    def get_current_millis():
        return round(time.time() * 1000)



"""
NX100 robot connection parameters
"""
NX100_IP_ADDRESS = "192.168.2.28"
NX100_TCP_PORT = 80

"""
When True, will return mock responses so no real robot is required
mostly used to run tests
"""
MOCK_RESPONSE = False



from nx100_remote_control.module import Commands
from nx100_remote_control.objects import IO, Gripper
import logging

'''
These signals are this repository end goal specific
following gripper and its signals are custom device not supplied by Motoman
dont run these commands with actual robot with other hardware connected into these signal registers
'''

GRIPPER_OPEN_CLOSE_SIGNAL_INPUT = 22010  # first network input signal (only input method)
GRIPPER_OPEN_CLOSE_SIGNAL = 10022  # Universal Output IO signal register
GRIPPER_ACKNOWLEDGE_SIGNAL = 25  # Universal Input IO signal register
GRIPPER_HIT_SIGNAL = 27  # Universal Input IO signal register


def write_gripper_close():
    io_out = IO.IO(GRIPPER_OPEN_CLOSE_SIGNAL_INPUT, 4)  # each signal line has 8 bits, write them all
    io_out.set_first_eight_io_contracts('00000100')  # Todo, value not decided
    return Commands.write_io_signals(io_out)


def write_gripper_open():
    io_out = IO.IO(GRIPPER_OPEN_CLOSE_SIGNAL_INPUT, 4)  # each signal line has 8 bits, write them all
    io_out.set_first_eight_io_contracts('00000000')  # Todo, value not decided
    return Commands.write_io_signals(io_out)


# gripper closed or open
def read_gripper_closed_command_register():
    gripper_status = Gripper.Gripper()
    gripper_status.set_closed_status(
        Commands.read_io_signals(IO.IO(GRIPPER_OPEN_CLOSE_SIGNAL, 4))
    )
    return gripper_status


# gripper close or open command ack
def read_gripper_acknowledge():
    gripper_status = Gripper.Gripper()
    res = Commands.read_io_signals(IO.IO(GRIPPER_ACKNOWLEDGE_SIGNAL, 1)).get_response()
    logging.info('Gripper ack decimal: ' + res)
    gripper_status.set_closed_status(res)
    return gripper_status.is_gripper_closed()


# same command as upper one but better function name
def gripper_is_closed():
    gripper_status = Gripper.Gripper()
    res = Commands.read_io_signals(IO.IO(GRIPPER_ACKNOWLEDGE_SIGNAL, 1)).get_response()
    logging.info('Gripper ack decimal: ' + res)
    gripper_status.set_closed_status(res)
    return gripper_status.is_gripper_closed()


# gripper hit something signal on or off (this should program set hold or eStop on)
def read_gripper_hit():
    gripper_status = Gripper.Gripper()
    gripper_status.set_closed_status(
        Commands.read_io_signals(IO.IO(GRIPPER_HIT_SIGNAL, 4))
    )
    return gripper_status



# normal style responses like 0000
class Response(object):

    # Class constructor
    def __init__(self, response):
        self.response = response

    def get_response(self):
        return self.response

    def is_success(self):
        return False if '0000' not in self.response else True



from nx100_remote_control.objects import Command, Status, JobDetail, CurrentPos, Alarm, Response
from nx100_remote_control.module import Socket, Utils
import logging
import time


# Reads the error alarm code
def read_alarms():
    request_alarms = Socket.exec_single_command(Command.Command("RALARM", ""))
    # Utils.print_response_details(request_alarms)
    alarms = Alarm.Alarm(request_alarms)
    logging.info('Alarms: ' + str(alarms.get_alarms()))
    return alarms


# Reads the current position in joint coordinate system
def read_current_joint_coordinate_position():
    response_data = Socket.exec_single_command(Command.Command("RPOSJ", ""))
    Utils.print_response_details(response_data)
    return response_data


# Reads the current position in a specified coordinate system.
# The specification with or without external axis can be made
# coordinate_system = 0: Base coordinate, 1: Robot coordinate, 2: User coordinate 1...24
def read_current_specified_coordinate_system_position(coordinate_system, include_external_axis='0'):
    position_response = Socket.exec_single_command(
        Command.Command("RPOSC", (coordinate_system + ', ' + include_external_axis))
    )
    Utils.print_response_details(position_response)
    return CurrentPos.CurrentPos(position_response)


# Reads the status of mode, cycle, operation, alarm error, and servo
def read_status():
    response_data = Socket.exec_single_command(Command.Command("RSTATS", ""))
    Utils.print_response_details(response_data)
    parts = response_data.split(',')
    data_1 = Utils.decimal_to_binary(int(parts[0]))
    data_2 = Utils.decimal_to_binary(int(parts[1]))
    s = Status.Status(data_1, data_2)
    logging.info('Command remote: ' + str(s.is_command_remote()) + ', ' +
                 'Play: ' + str(s.is_play()) + ', ' +
                 'Teach ' + str(s.is_teach()) + ', ' +
                 'Safety speed operation: ' + str(s.is_safety_speed_operation()) + ', ' +
                 'Running: ' + str(s.is_running()) + ', ' +
                 'Auto: ' + str(s.is_auto()) + ', ' +
                 'One cycle: ' + str(s.is_one_cycle()) + ', ' +
                 'Step: ' + str(s.is_step()) + ', ' +
                 'Servo on: ' + str(s.is_servo_on()) + ', ' +
                 'Error occurring: ' + str(s.is_error_occurring()) + ', ' +
                 'Alarm occurring: ' + str(s.is_alarm_occurring()) + ', ' +
                 'Command hold: ' + str(s.is_command_hold()) + ', ' +
                 'External hold: ' + str(s.is_external_hold()) + ', ' +
                 'Programming pendant hold: ' + str(s.is_programming_pendant_hold()))
    return s


# Reads the current job name, line No. and step No
def read_current_job_details():
    response_data = Socket.exec_single_command(Command.Command("RJSEQ", ""))
    Utils.print_response_details(response_data)
    return JobDetail.JobDetail(response_data)


# Turns HOLD ON/OFF
def write_hold(command):
    if command not in '1' and command not in '0':
        logging.error('[E] hold command can only be 1 (on) or 0 (off)')
        return
    response = Response.Response(Socket.exec_single_command(Command.Command("HOLD", command)))
    Utils.print_response_details(response.get_response())
    logging.info('[i] hold command run successfully!' if response.is_success() else '[E] hold command run failed!')
    return response


# Resets an alarm of manipulator
def write_reset():
    response = Response.Response(Socket.exec_single_command(Command.Command("RESET", "")))
    Utils.print_response_details(response.get_response())
    logging.info('[i] reset command run successfully!' if response.is_success() else '[E] reset command run failed!')
    return response


# Cancels an error
def write_cancel():
    response = Response.Response(Socket.exec_single_command(Command.Command("CANCEL", "")))
    Utils.print_response_details(response.get_response())
    logging.info('[i] cancel command run successfully!' if response.is_success() else '[E] cancel command run failed!')
    return response


# Turns servo power supply ON/OFF
def write_servo_power(command):
    if command not in '1' and command not in '0':
        logging.error('[E] servo power command can only be 1 (on) or 0 (off)')
        return
    response = Response.Response(Socket.exec_single_command(Command.Command("SVON", command)))
    Utils.print_response_details(response.get_response())
    logging.info('[i] servo command run successfully!' if response.is_success() else '[E] servo command run failed!')
    return response


# Starts a job
def write_start_job(job_name):
    response = Response.Response(Socket.exec_single_command(Command.Command("START", job_name)))
    Utils.print_response_details(response.get_response())
    logging.info(
        '[i] start job command run successfully!' if response.is_success() else '[E] start job command run failed!')
    return response


# Moves a manipulator to a specified coordinate position in linear motion
def write_linear_move(move_l):
    move_cmd = move_l.get_command()
    logging.info('[i] move: ' + move_cmd)
    response = Response.Response(Socket.exec_single_command(
        Command.Command("MOVL", move_cmd)
    ))
    # Utils.print_response_details(response_data)
    logging.info('[i] command run successfully!' if response.is_success() else '[E] command run failed!')
    return response


# Moves a manipulator to a specified coordinate position in joint motion.
def write_joint_motion_move(move_j):
    move_cmd = move_j.get_command()
    logging.info('[i] move: ' + move_cmd)
    response = Response.Response(Socket.exec_single_command(
        Command.Command("MOVJ", move_cmd)
    ))
    logging.info('[i] command run successfully!' if response.is_success() else '[E] command run failed!')
    return response


# Is robot in target point function with callback. Timeout given in seconds
def robot_in_target_point_callback(move_l, timeout=10, _callback_success=None, _callback_failed=None):
    current = 0
    for x in range(timeout):
        time.sleep(1)
        cp = read_current_specified_coordinate_system_position(  # returns CurrentPos object
            str(move_l.get_coordinate_specification), '0'
        )
        if Utils.is_in_position(move_l, cp):
            if _callback_success:
                _callback_success()
                break
            else:
                return True
        else:
            current = current + 1
            if current == timeout:
                if _callback_failed:
                    _callback_failed()
                    break
                else:
                    return False


# Reads I/O signals, contact point No. to start read-out, the number of contact points to be read out
def read_io_signals(io):
    response = Response.Response(Socket.exec_single_command(
        Command.Command("IOREAD", io.get_io_read_command())
    ))
    Utils.print_response_details(response.get_response())
    logging.info(
        '[i] IO read command run successfully!' if response.is_success() else '[E] IO read command run failed!')
    return response


# Write I/O signals
def write_io_signals(io):
    response = Response.Response(Socket.exec_single_command(
        Command.Command("IOWRITE", io.get_io_write_command()))
    )
    Utils.print_response_details(response.get_response())
    logging.info(
        '[i] IO write command run successfully!' if response.is_success() else '[E] IO write command run failed!')
    return response


# Read all registered job names
def read_all_job_names():
    response = Response.Response(Socket.exec_single_command(
        Command.Command("RJDIR", "*"))  # * means to read all registered jobs
    )
    Utils.print_response_details(response.get_response())
    logging.info('[i] Read job names command run successfully!' if response.is_success()
                 else '[E] Read job names command run failed!')
    return response


# Set wanted job as master job
def write_master_job(job_name):
    response = Response.Response(Socket.exec_single_command(Command.Command("SETMJ", job_name)))
    Utils.print_response_details(response.get_response())
    logging.info('[i] set master job command run successfully!' if response.is_success()
                 else '[E] set master job command run failed!')
    return response



class Alarm(object):

    # Class constructor
    def __init__(self, response):
        self.alarms = str(response).split(',')

    # return alarm from wanted position
    def get_alarm(self, position=0):
        return self.alarms[position] if position < 10 else 'N/A'

    def get_alarms(self):
        return self.alarms


"""
IO network input and output range is start 22010 and ends to signal 23287
"""


class IO(object):

    # Class constructor
    def __init__(self, contact_point_start_no, num_of_contact_points=8):
        self.contact_point_start_no = contact_point_start_no
        self.num_of_contact_points = num_of_contact_points
        self.first_eight_contacts_in = ''

    def get_io_read_command(self):
        return str(self.contact_point_start_no) + ', ' + str(self.num_of_contact_points)

    # Input example, '00000000'
    def set_first_eight_io_contracts(self, eight_contacts_data):
        self.first_eight_contacts_in = eight_contacts_data

    def get_io_write_command(self):
        return str(self.contact_point_start_no) + ', ' + \
               str(self.num_of_contact_points) + ', ' + \
               str(self.first_eight_contacts_in)



# noinspection SpellCheckingInspection
RALARM = "b'0,0,0,0,0,0,0,0,0,0\\r'"

# noinspection SpellCheckingInspection
RPOSJ = "b'45411,-59646,-55567,667,-1077,-1260,0,0,0,0,0,0\\r'"

# noinspection SpellCheckingInspection
RPOSC = "b'353.769,202.779,120.658,-1.34,35.78,27.84,0,0\\r'"
'''
Base coordinate:    b'353.769,202.779,120.658,-1.34,35.78,27.84,0,0\\r'
Robot coordinate:   b'353.769,202.779,120.658,-1.34,35.78,27.84,0,0\\r'
User coordinate:    b'640.625,274.481,353.192,-1.36,35.77,-107.58,0,0\\r'
'''

# noinspection SpellCheckingInspection
RSTATS = "b'194,0\\r'"

# noinspection SpellCheckingInspection
RJSEQ = "b'POYTA,10,9\\r'"

# noinspection SpellCheckingInspection
HOLD = "b'0000\\r\\n'"

# noinspection SpellCheckingInspection
RESET = "b'0000\\r\\n'"

# noinspection SpellCheckingInspection
CANCEL = "b'0000\\r\\n'"

# noinspection SpellCheckingInspection
SVON = "b'0000\\r\\n'"

# noinspection SpellCheckingInspection
START = "b'0000\\r\\n'"

# noinspection SpellCheckingInspection
MOVL = "b'0000\\r\\n'"

# noinspection SpellCheckingInspection
MOVJ = "b'0000\\r\\n'"

# noinspection SpellCheckingInspection
IOWRITE = "b'0\\r\\n'"

# noinspection SpellCheckingInspection
IOREAD = "b'0\\r\\n'"

# noinspection SpellCheckingInspection
RJDIR = ""

# noinspection SpellCheckingInspection
SETMJ = "b'0000\\r\\n'"


def get_mock_response(command):
    if command.name == 'RALARM':
        return RALARM
    elif command.name == 'RPOSJ':
        return RPOSJ
    elif command.name == 'RPOSC':
        return RPOSC
    elif command.name == 'RSTATS':
        return RSTATS
    elif command.name == 'RJSEQ':
        return RJSEQ
    elif command.name == 'HOLD':
        return HOLD
    elif command.name == 'RESET':
        return RESET
    elif command.name == 'CANCEL':
        return CANCEL
    elif command.name == 'SVON':
        return SVON
    elif command.name == 'START':
        return START
    elif command.name == 'MOVL':
        return MOVL
    elif command.name == 'MOVJ':
        return MOVJ
    elif command.name == 'IOWRITE':
        return IOWRITE
    elif command.name == 'IOREAD':
        return IOREAD
    elif command.name == 'RJDIR':
        return RJDIR
    elif command.name == 'SETMJ':
        return SETMJ
    else:
        return '[E] no mock response for given command ' + command.name



class JobDetail(object):

    # Class constructor
    def __init__(self, response):
        self.parts = response.split(',')

    def job_name(self):
        p = 0
        return self.parts[p] if len(self.parts) > p else 'Job name N/A'

    def line_no(self):
        p = 1
        return self.parts[p] if len(self.parts) > p else 'N/A'

    def step_no(self):
        p = 2
        return self.parts[p] if len(self.parts) > p else 'N/A'



from setuptools import setup

setup(
    name='nx100_remote_control',
    packages=[
        'nx100_remote_control',
        'nx100_remote_control.module',
        'nx100_remote_control.objects'
    ],
    package_dir={'': 'src'},
    include_package_data=True,
)



class MoveL(object):
    # https://drive.google.com/file/d/11TY9v_Tb5k23DTz9VuEBmj-vJE5Fmc4R/view

    # motion_speed_selection options
    motion_speed_selection_post_speed = 0  # this option will do fast angle speeds
    motion_speed_selection_posture_speed = 1  # this option does smoother angle speeds defined by motion_speed variable

    # coordinate_specification options
    coordinate_specification_base_coordinate = 0  # base coordinate option
    coordinate_specification_robot_coordinate = 1  # robot coordinate option
    coordinate_specification_user_1_coordinate = 2  # user 1 coordinate option

    # Class constructor
    def __init__(self, motion_speed_selection, motion_speed, coordinate_specification,
                 x, y, z, tx, ty, tz, d_10, d_11=0, d_12=0, d_13=0, d_14=0, d_15=0, d_16=0, d_17=0):
        self.motion_speed_selection = motion_speed_selection
        self.motion_speed = motion_speed
        self.coordinate_specification = coordinate_specification
        self.x = x
        self.y = y
        self.z = z
        self.tx = tx
        self.ty = ty
        self.tz = tz
        self.d_10 = d_10
        self.d_11 = d_11
        self.d_12 = d_12
        self.d_13 = d_13
        self.d_14 = d_14
        self.d_15 = d_15
        self.d_16 = d_16
        self.d_17 = d_17

    def get_command(self):
        separator = ', '
        return separator.join([
            str(self.motion_speed_selection),
            str(self.motion_speed),
            str(self.coordinate_specification),
            str(self.x),
            str(self.y),
            str(self.z),
            str(self.tx),
            str(self.ty),
            str(self.tz),
            str(self.d_10),
            str(self.d_11),
            str(self.d_12),
            str(self.d_13),
            str(self.d_14),
            str(self.d_15),
            str(self.d_16),
            str(self.d_17)
        ])

    def get_coordinate_specification(self):
        return self.coordinate_specification

    def get_x(self):
        return self.x

    def get_y(self):
        return self.y

    def get_z(self):
        return self.z

    def get_tx(self):
        return self.tx

    def get_ty(self):
        return self.ty

    def get_tz(self):
        return self.tz



import nx100_remote_control
from nx100_remote_control.module import Utils, MockResponse
import logging
import socket

CR = "\r"
CRLF = "\r\n"


# exec single command object
def exec_single_command(command):
    if nx100_remote_control.MOCK_RESPONSE:
        return Utils.clean_response(MockResponse.get_mock_response(command))

    client = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    client.settimeout(5)

    # connect the client
    client.connect((nx100_remote_control.NX100_IP_ADDRESS, nx100_remote_control.NX100_TCP_PORT))

    # start request
    start_request = "CONNECT Robot_access" + CRLF
    client.send(start_request.encode())
    response = client.recv(4096)
    start_response = repr(response)
    if 'OK: NX Information Server' not in start_response:
        client.close()
        logging.error('[E] Command start request response not ok!')
        return

    # command request
    cmd_data_length = Utils.command_data_length(command)
    command_request = "HOSTCTRL_REQUEST " + command.name + " " + str(cmd_data_length) + CRLF
    logging.info('Sending command: ' + command_request)
    client.send(command_request.encode())
    response = client.recv(4096)
    command_response = repr(response)
    if ('OK: ' + command.name) not in command_response:
        client.close()
        logging.error('[E] Command request response not ok!')
        return

    # command data request
    command_data_request = command.data + (CR if len(command.data) > 0 else '')
    client.send(command_data_request.encode())
    response = client.recv(4096)
    command_data_response = repr(response)

    # close socket
    client.close()

    return Utils.clean_response(command_data_response)


# exec array of command objects
def exec_multiple_commands(commands):
    logging.info('No implementation')



from http.server import HTTPServer, BaseHTTPRequestHandler
from nx100_remote_control.objects import MoveL
from nx100_remote_control.module import Commands, Utils
import logging
import os

base_path = os.path.dirname(__file__)

SPEED = 50
MOVE_MM = 25


def get_position():
    return Commands.read_current_specified_coordinate_system_position('0', '0')


# noinspection PyPep8Naming
class S(BaseHTTPRequestHandler):
    def _set_headers(self):
        self.send_response(200)
        self.send_header("Content-type", "text/html")
        self.end_headers()

    def _html(self, message):
        try:
            if str(self.path) == '/':
                filename = 'index.html'
                html_content = f"<html><body><h1>{message}</h1></body></html>"
                job_name = Commands.read_current_job_details().job_name()
                status = Commands.read_status()
                c_pos = get_position()
                with open(os.path.join(base_path, filename)) as f:
                    html_content = f.read()
                    html_content = html_content \
                        .replace('{{jobName}}', job_name) \
                        .replace('{{commandRemote}}', str(status.is_command_remote())) \
                        .replace('{{playMode}}', str(status.is_play())) \
                        .replace('{{isHold}}', str(
                        status.is_command_hold() or status.is_command_hold() or status.is_programming_pendant_hold())) \
                        .replace('{{teachMode}}', str(status.is_teach())) \
                        .replace('{{running}}', str(status.is_running())) \
                        .replace('{{servoOn}}', str(status.is_servo_on())) \
                        .replace('{{isError}}', str(status.is_error_occurring())) \
                        .replace('{{x}}', str(c_pos.x)) \
                        .replace('{{y}}', str(c_pos.y)) \
                        .replace('{{z}}', str(c_pos.z)) \
                        .replace('{{tx}}', str(c_pos.tx)) \
                        .replace('{{ty}}', str(c_pos.ty)) \
                        .replace('{{tz}}', str(c_pos.tz))
                return html_content.encode("utf8")  # NOTE: must return a bytes object!
            else:
                return "".encode('utf-8')
        except Exception as e:
            return html_content.encode("utf8")

    def do_GET(self):
        self._set_headers()
        self.wfile.write(self._html("Problem with robot connection"))

    def do_HEAD(self):
        self._set_headers()

    def do_POST(self):
        content_length = int(self.headers['Content-Length'])  # <--- Gets the size of data
        post_data = self.rfile.read(content_length)  # <--- Gets the data itself
        # logging.info("POST request,\nPath: %s\nHeaders:\n%s\n\nBody:\n%s\n", str(self.path), str(self.headers),
        #             post_data.decode('utf-8'))
        command = post_data.decode("utf-8")
        if command == 'start_job':
            Commands.write_start_job('')
        elif command == 'hold_on':
            Commands.write_hold('1')
        elif command == 'hold_off':
            Commands.write_hold('0')
        elif command == 'servo_on':
            Commands.write_servo_power('1')
        elif command == 'servo_off':
            Commands.write_servo_power('0')
        elif command == 'move_s_l' or command == 'move_s_r':
            c_pos = get_position()
            Commands.write_linear_move(MoveL.MoveL(
                0, SPEED, 0,
                (c_pos.get_x() - MOVE_MM) if command == 'move_s_l' else (c_pos.get_x() + MOVE_MM),
                c_pos.get_y(),
                c_pos.get_z(), c_pos.get_tx(), c_pos.get_ty(), c_pos.get_tz(),
                Utils.binary_to_decimal(0x00000001)
            ))
        elif command == 'move_l_l' or command == 'move_l_r':
            c_pos = get_position()
            Commands.write_linear_move(MoveL.MoveL(
                0, SPEED, 0,
                c_pos.get_x(),
                (c_pos.get_y() - MOVE_MM) if command == 'move_l_l' else (c_pos.get_y() + MOVE_MM),
                c_pos.get_z(), c_pos.get_tx(), c_pos.get_ty(), c_pos.get_tz(),
                Utils.binary_to_decimal(0x00000001)
            ))
        elif command == 'move_u_l' or command == 'move_u_r':
            c_pos = get_position()
            Commands.write_linear_move(MoveL.MoveL(
                0, SPEED, 0,
                c_pos.get_x(), c_pos.get_y(),
                (c_pos.get_z() - MOVE_MM) if command == 'move_u_l' else (c_pos.get_z() + MOVE_MM),
                c_pos.get_tx(), c_pos.get_ty(), c_pos.get_tz(),
                Utils.binary_to_decimal(0x00000001)
            ))
        # Todo, do not use, ultra dangerous
        # Todo, implement these some day with posture speed option
        # elif command == 'move_r_l' or command == 'move_r_r':
        #     c_pos = get_position()
        #     Commands.write_linear_move(MoveL.MoveL(
        #         0, 1, 0,
        #         c_pos.get_x(), c_pos.get_y(), c_pos.get_z(),
        #         (c_pos.get_tx() - 5.00) if command == 'move_r_l' else (c_pos.get_tx() + 5.0),
        #         c_pos.get_ty(), c_pos.get_tz(),
        #         Utils.binary_to_decimal(0x00000001)
        #     ))

        self._set_headers()
        self.wfile.write("".format(self.path).encode('utf-8'))


def run(server_class=HTTPServer, handler_class=S, addr="localhost", port=8080):
    server_address = (addr, port)
    httpd = server_class(server_address, handler_class)

    logging.info(f"Starting httpd server on {addr}:{port}")
    httpd.serve_forever()



#  Copyright (c) 2017 Jon Cooper
#
#  This file is part of pygame-xbox360controller.
#  Documentation, related files, and licensing can be found at
#
#      <https://github.com/joncoop/pygame-xbox360controller>.


import pygame
import sys

LINUX = 0
MAC = 1
WINDOWS = 2

if sys.platform.startswith("lin"):
    platform_id = LINUX
elif sys.platform.startswith("darwin"):
    platform_id = MAC
elif sys.platform.startswith("win"):
    platform_id = WINDOWS

if platform_id == LINUX:
    # buttons
    A = 0
    B = 1
    X = 2
    Y = 3
    LEFT_BUMP = 4
    RIGHT_BUMP = 5
    BACK = 6
    START = 7
    # GUIDE = 8
    LEFT_STICK_BTN = 9
    RIGHT_STICK_BTN = 10

    # axes
    LEFT_STICK_X = 0
    LEFT_STICK_Y = 1
    RIGHT_STICK_X = 3
    RIGHT_STICK_Y = 4
    LEFT_TRIGGER = 2
    RIGHT_TRIGGER = 5

elif platform_id == WINDOWS:
    # buttons
    A = 0
    B = 1
    X = 2
    Y = 3
    LEFT_BUMP = 4
    RIGHT_BUMP = 5
    BACK = 6
    START = 7
    LEFT_STICK_BTN = 8
    RIGHT_STICK_BTN = 9

    # axes
    LEFT_STICK_X = 0
    LEFT_STICK_Y = 1
    RIGHT_STICK_X = 4
    RIGHT_STICK_Y = 3
    TRIGGERS = 2

elif platform_id == MAC:
    # buttons
    A = 11
    B = 12
    X = 13
    Y = 14
    LEFT_BUMP = 8
    RIGHT_BUMP = 9
    BACK = 5
    START = 4
    LEFT_STICK_BTN = 6
    RIGHT_STICK_BTN = 7

    # d-pad
    PAD_UP = 0
    PAD_DOWN = 1
    PAD_LEFT = 2
    PAD_RIGHT = 3

    # axes
    LEFT_STICK_X = 0
    LEFT_STICK_Y = 1
    RIGHT_STICK_X = 2
    RIGHT_STICK_Y = 3
    LEFT_TRIGGER = 4
    RIGHT_TRIGGER = 5


class Controller:
    id_num = 0

    def __init__(self, dead_zone=0.15):
        """
        Initializes a controller. IDs for controllers begin at 0 and increment by 1
        each time a controller is initialized.

        Args:
            dead_zone: The size of dead zone for the analog sticks (default 0.15)
        """

        pygame.joystick.init()

        self.joystick = pygame.joystick.Joystick(Controller.id_num)
        self.joystick.init()
        self.dead_zone = dead_zone

        # Linux and Mac triggers behave funny. See get_triggers().
        self.left_trigger_used = False
        self.right_trigger_used = False

        Controller.id_num += 1

    def get_id(self):
        """
        Returns:
            The ID of the controller.
        """

        return self.joystick.get_id()

    def dead_zone_adjustment(self, value):
        """
        Analog sticks likely wont ever return to exact center when released. Without
        a dead zone, it is likely that a small axis value will cause game objects
        to drift. This adjusment allows for a full range of input while still
        allowing a little bit of 'play' in the dead zone.

        Returns:
            Axis value outside of the dead zone remapped proportionally onto the
            -1.0 <= value <= 1.0 range.
        """

        if value > self.dead_zone:
            return (value - self.dead_zone) / (1 - self.dead_zone)
        elif value < -self.dead_zone:
            return (value + self.dead_zone) / (1 - self.dead_zone)
        else:
            return 0

    def get_buttons(self):
        """
        Gets the state of each button on the controller.

        Returns:
            A tuple with the state of each button. 1 is pressed, 0 is unpressed.
        """

        if platform_id == LINUX:
            return (self.joystick.get_button(A),
                    self.joystick.get_button(B),
                    self.joystick.get_button(X),
                    self.joystick.get_button(Y),
                    self.joystick.get_button(LEFT_BUMP),
                    self.joystick.get_button(RIGHT_BUMP),
                    self.joystick.get_button(BACK),
                    self.joystick.get_button(START),
                    0,  # Unused, since Guide only works on Linux
                    self.joystick.get_button(LEFT_STICK_BTN),
                    self.joystick.get_button(RIGHT_STICK_BTN))

        elif platform_id == WINDOWS:
            return (self.joystick.get_button(A),
                    self.joystick.get_button(B),
                    self.joystick.get_button(X),
                    self.joystick.get_button(Y),
                    self.joystick.get_button(LEFT_BUMP),
                    self.joystick.get_button(RIGHT_BUMP),
                    self.joystick.get_button(BACK),
                    self.joystick.get_button(START),
                    self.joystick.get_button(LEFT_STICK_BTN),
                    self.joystick.get_button(RIGHT_STICK_BTN))

        elif platform_id == MAC:
            return (0,  # Unused
                    0,  # Unused
                    0,  # Unused
                    0,  # Unused
                    self.joystick.get_button(START),
                    self.joystick.get_button(BACK),
                    self.joystick.get_button(LEFT_STICK_BTN),
                    self.joystick.get_button(RIGHT_STICK_BTN),
                    self.joystick.get_button(LEFT_BUMP),
                    self.joystick.get_button(RIGHT_BUMP),
                    0,  # Unused
                    self.joystick.get_button(A),
                    self.joystick.get_button(B),
                    self.joystick.get_button(X),
                    self.joystick.get_button(Y))

    def get_left_stick(self):
        """
        Gets the state of the left analog stick.

        Returns:
            The x & y axes as a tuple such that

            -1 <= x <= 1 && -1 <= y <= 1

            Negative values are left and up.
            Positive values are right and down.
        """

        left_stick_x = self.dead_zone_adjustment(self.joystick.get_axis(LEFT_STICK_X))
        left_stick_y = self.dead_zone_adjustment(self.joystick.get_axis(LEFT_STICK_Y))

        return (left_stick_x, left_stick_y)

    def get_right_stick(self):
        """
        Gets the state of the right analog stick.

        Returns:
            The x & y axes as a tuple such that

            -1 <= x <= 1 && -1 <= y <= 1

            Negative values are left and up.
            Positive values are right and down.
        """

        right_stick_x = self.dead_zone_adjustment(self.joystick.get_axis(RIGHT_STICK_X))
        right_stick_y = self.dead_zone_adjustment(self.joystick.get_axis(RIGHT_STICK_Y))

        return (right_stick_x, right_stick_y)

    def get_triggers(self):
        """
        Gets the state of the triggers.

        On Windows, both triggers work additively to return a single axis, whereas
        triggers on Linux and Mac function as independent axes. In this interface,
        triggers will behave additively for all platforms so that pygame controllers
        will work consistently on each platform.

        Also note that the value returned is on Windows is multiplied by -1 so that
        negative is to the left and positive to the right to be consistent with
        stick axes.

        On Linux and Mac, trigger axes return 0 if they haven't been used yet. Once
        used, an unpulled trigger returns 1 and pulled returns -1. The trigger_used
        booleans keep the math correct for triggers prior to use.

        Returns:
            A float in the range -1.0 <= value <= 1.0 where -1.0 represents full
            left and 1.0 represents full right. If the triggers are pulled
            simultaneously, then the sum of the trigger pulls is returned.
        """

        trigger_axis = 0.0

        if platform_id == LINUX or platform_id == MAC:
            left = self.joystick.get_axis(LEFT_TRIGGER)
            right = self.joystick.get_axis(RIGHT_TRIGGER)

            if left != 0:
                self.left_trigger_used = True
            if right != 0:
                self.right_trigger_used = True

            if not self.left_trigger_used:
                left = -1
            if not self.right_trigger_used:
                right = -1

            trigger_axis = (-1 * left + right) / 2

        elif platform_id == WINDOWS:
            trigger_axis = -1 * self.joystick.get_axis(TRIGGERS)

        return trigger_axis

    def get_pad(self):
        """
        Gets the state of the directional pad.

        Returns:
            A tuple in the form (up, right, down, left) where each value will be
            1 if pressed, 0 otherwise. Pads are 8-directional, so it is possible
            to have up to two 1s in the returned tuple.
        """

        if platform_id == LINUX or platform_id == WINDOWS:
            hat_x, hat_y = self.joystick.get_hat(0)

            up = int(hat_y == 1)
            right = int(hat_x == 1)
            down = int(hat_y == -1)
            left = int(hat_x == -1)

        elif platform_id == MAC:
            up = self.joystick.get_button(PAD_UP)
            right = self.joystick.get_button(PAD_RIGHT)
            down = self.joystick.get_button(PAD_DOWN)
            left = self.joystick.get_button(PAD_LEFT)

        return up, right, down, left
